#!/usr/bin/env python
# coding: utf-8
import subprocess
import numpy as np
import sympy as sp
import pystencils as ps
from pystencils_benchmark import generate_benchmark, Compiler
from pathlib import Path


def generate(path: Path, compiler: Compiler):
    a, b, c = ps.fields(a=np.ones(4000000), b=np.ones(4000000), c=np.ones(4000000))
    alpha = sp.symbols('alpha')

    kernels = []
    @ps.kernel_config(ps.CreateKernelConfig())
    def vadd():
        a[0] @= b[0] + c[0]
    kernels.append(ps.create_kernel(**vadd))

    @ps.kernel_config(ps.CreateKernelConfig(cpu_vectorize_info={'instruction_set': 'best'}))
    def vadd_vector():
        a[0] @= b[0] + c[0]
    kernels.append(ps.create_kernel(**vadd_vector))

    @ps.kernel_config(ps.CreateKernelConfig())
    def daxpy():
        b[0] @= alpha * a[0] + b[0]
    kernels.append(ps.create_kernel(**daxpy))

    @ps.kernel_config(ps.CreateKernelConfig(cpu_vectorize_info={'instruction_set': 'best'}))
    def daxpy_vector():
        b[0] @= alpha * a[0] + b[0]
    kernels.append(ps.create_kernel(**daxpy_vector))

    generate_benchmark(kernels, path, compiler=compiler)


def make(path: Path):
    subprocess.run(['make'], check=True)


def execute(path: Path, compiler: Compiler):
    subprocess.run([f'./benchmark-{compiler.name}', '100'], check=True)


def main():
    compiler = Compiler.GCCdebug
    path = Path.cwd()
    generate(path, compiler)
    make(path)
    execute(path, compiler)


if __name__ == '__main__':
    main()

